/**
 * Check if an agent has already seen the intro message.
 * This is a synchronous operation for use in formatHelp.
 * Returns true on error to avoid blocking CLI (assumes seen).
 */
export declare function hasAgentSeenIntro(agentId: string): boolean;
/**
 * Mark an agent as having seen the intro message.
 * This is a synchronous operation for use in formatHelp.
 * Silently fails on error (non-critical feature).
 */
export declare function markAgentIntroSeen(agentId: string): void;
/**
 * Check if an agent has already seen the structured input hint.
 * Returns true on error to avoid blocking CLI (assumes seen).
 */
export declare function hasAgentSeenInputHint(agentId: string): boolean;
/**
 * Mark an agent as having seen the structured input hint.
 * Silently fails on error (non-critical feature).
 */
export declare function markAgentInputHintSeen(agentId: string): void;
//# sourceMappingURL=agent-intro.d.ts.map