export declare const symbols: {
    active: string;
    completed: string;
    error: string;
    warning: string;
    cancel: string;
    bar: string;
    barH: string;
    cornerTL: string;
    cornerTR: string;
    cornerBL: string;
    cornerBR: string;
    squareTL: string;
    squareTR: string;
    squareBL: string;
    squareBR: string;
    connect: string;
    connectRight: string;
    radioActive: string;
    radioInactive: string;
    checkboxActive: string;
    checkboxSelected: string;
    checkboxInactive: string;
    passwordMask: string;
    info: string;
};
export declare const isUnicode: boolean;
//# sourceMappingURL=symbols.d.ts.map