export declare const embeddingsSubcommand: import("../../..").CommandDefinition;
export declare const imageSubcommand: import("../../..").CommandDefinition;
export declare const speechSubcommand: import("../../..").CommandDefinition;
export declare const transcriptionSubcommand: import("../../..").CommandDefinition;
export declare const videoSubcommand: import("../../..").CommandDefinition;
//# sourceMappingURL=modalities.d.ts.map