import { ProjectConfig } from '../../../types';
export declare function resolveProjectId(ctx: {
    project?: ProjectConfig;
}, options: {
    projectId?: string;
}): string;
//# sourceMappingURL=utils.d.ts.map