/**
 * String utility functions for identifier conversion
 * Re-exported from @agentuity/core for convenience
 */
export { toCamelCase, toPascalCase } from '@agentuity/core';
//# sourceMappingURL=string.d.ts.map