import { z } from 'zod';
export declare const DeployRolloutMetadataSchema: z.ZodObject<{
    source: z.ZodOptional<z.ZodEnum<{
        managed: "managed";
    }>>;
    channel: z.ZodOptional<z.ZodEnum<{
        commit: "commit";
        edge: "edge";
        stable: "stable";
    }>>;
    rollout_org_ids: z.ZodOptional<z.ZodArray<z.ZodString>>;
}, z.core.$strict>;
export type DeployRolloutMetadata = z.infer<typeof DeployRolloutMetadataSchema>;
export declare function parseDeployRolloutMetadata(raw: string | undefined): DeployRolloutMetadata | undefined;
export declare function resolveDeployRolloutMetadata(options?: {
    metadata?: string;
}): DeployRolloutMetadata | undefined;
export declare function mergeDeployRolloutMetadata<T extends {
    deployment?: Record<string, unknown>;
}>(build: T, rolloutMetadata: DeployRolloutMetadata | undefined): T;
//# sourceMappingURL=deploy-rollout-metadata.d.ts.map