/**
 * Simple formatter for schema code strings.
 * Adds basic indentation and line breaks for readability.
 */
export declare function formatSchemaCode(code: string): string;
//# sourceMappingURL=format-schema.d.ts.map