/**
 * Agent Discovery — import-based
 *
 * Discovers agents by scanning src/agent/**\/*.ts files and importing them
 * at build time. The agent instance already knows its own metadata, schemas,
 * and evals — no AST parsing needed.
 */
import type { Logger } from '../../../types';
export interface AgentMetadata {
    filename: string;
    name: string;
    id: string;
    agentId: string;
    version: string;
    description?: string;
    inputSchemaCode?: string;
    outputSchemaCode?: string;
    evals?: EvalMetadata[];
}
export interface EvalMetadata {
    id: string;
    identifier: string;
    name: string;
    filename: string;
    version: string;
    description?: string;
    agentIdentifier: string;
    projectId: string;
}
/**
 * Discover all agents in src/agent directory.
 *
 * Imports each agent file at build time — the agent instance already knows
 * its own metadata, schemas, and evals. No AST parsing needed.
 */
export declare function discoverAgents(srcDir: string, projectId: string, deploymentId: string, logger: Logger): Promise<AgentMetadata[]>;
//# sourceMappingURL=agent-discovery.d.ts.map