/**
 * Documentation Generator
 *
 * Generates README.md and AGENTS.md in src/generated/ directory
 * Only writes files if they don't already exist (preserves user modifications)
 */
import type { Logger } from '../../../types';
/**
 * Generate documentation files in src/generated/ directory
 * Only creates files if they don't already exist
 */
export declare function generateDocumentation(srcDir: string, logger: Logger): Promise<void>;
//# sourceMappingURL=docs-generator.d.ts.map