export type ExtensionPathResolverOptions = {
    cwd?: string;
    env?: NodeJS.ProcessEnv;
    moduleUrl?: string;
};
export declare function resolveExtensionPath(flagPath?: string, options?: ExtensionPathResolverOptions): Promise<string | null>;
export declare function resolveExtensionRuntimeModulePath(extensionPath: string): Promise<string | null>;
//# sourceMappingURL=extension-path.d.ts.map