/**
 * Build-time patches for OpenTelemetry LLM instrumentation.
 *
 * These patches wrap LLM SDK methods (OpenAI, Anthropic, etc.) with OTel spans
 * at build time, since runtime instrumentation (traceloop) doesn't work with
 * bundled code.
 */
import { type PatchModule } from './_util';
export declare function generatePatches(): Map<string, PatchModule>;
//# sourceMappingURL=otel-llm.d.ts.map