import { Logger } from '@agentuity/core';
export interface TemplateInfo {
    id: string;
    name: string;
    description: string;
    directory: string;
}
export declare function fetchTemplates(logger: Logger, localDir?: string, branch?: string): Promise<TemplateInfo[]>;
//# sourceMappingURL=templates.d.ts.map