import type { Logger } from '@agentuity/core';
import type { APIClient } from '../../../api';
import type { Config } from '../../../types';
export interface RunGitIdentityConnectOptions {
    apiClient: APIClient;
    logger: Logger;
    config?: Config | null;
}
export interface RunGitIdentityConnectResult {
    connected: boolean;
    cancelled?: boolean;
}
export declare function runGitIdentityConnect(options: RunGitIdentityConnectOptions): Promise<RunGitIdentityConnectResult>;
export declare const connectSubcommand: import("../../..").SubcommandDefinition;
//# sourceMappingURL=connect.d.ts.map