/**
 * Generate the introduction prompt for AI agents
 */
export declare function generateIntroPrompt(agent: string | undefined): string;
export declare const introSubcommand: import("../..").SubcommandDefinition;
export default introSubcommand;
//# sourceMappingURL=intro.d.ts.map