import { type AIGatewayModels } from '@agentuity/core';
export declare function getCachedAIGatewayModels(profile: string, cacheKey: string): Promise<AIGatewayModels | null>;
export declare function setCachedAIGatewayModels(profile: string, cacheKey: string, models: AIGatewayModels): Promise<void>;
//# sourceMappingURL=model-cache.d.ts.map