/**
 * Get the catalyst URL for a specific region.
 * The catalyst can resolve any prefixed ID (sbx_, proj_, org_, etc.)
 * without requiring org context.
 */
export declare function getCatalystUrl(region: string, overrides?: {
    catalyst_url?: string;
} | null): string;
//# sourceMappingURL=catalyst.d.ts.map