export declare const colors: {
    active: (text: string) => string;
    completed: (text: string) => string;
    error: (text: string) => string;
    warning: (text: string) => string;
    success: (text: string) => string;
    info: (text: string) => string;
    muted: (text: string) => string;
    bold: (text: string) => string;
    underline: (text: string) => string;
    reset: (text: string) => string;
    primary: (text: string) => string;
    secondary: (text: string) => string;
    link: (text: string) => string;
    inverseCyan: (text: string) => string;
};
//# sourceMappingURL=colors.d.ts.map